<?xml version="1.0" encoding="UTF-8"?>
<playlist version="1" xmlns="http://xspf.org/ns/0/">
	<trackList>
		<track>
			<title><![CDATA[12:15 Terrorattack avvärjd i Australien]]></title>
			<location>http://www0.c00928.cdn.qbrick.com/00928/kluster/20090804/PR-2009-0804-SOBEAUSTRALIEN.flv</location>
			<duration>73000</duration>
			<image>http://media.svt.se/download/mcc/flash/20090804/PR-2009-0804-SOBEAUSTRALIEN/PR-2009-0804-SOBEAUSTRALIEN_start_0.jpg</image>
		</track>
		<track>
			<title><![CDATA[12:12 Raketattack mot Kabul]]></title>
			<location>http://www0.c00928.cdn.qbrick.com/00928/kluster/20090804/PR-2009-0804-SOBEKABUL.flv</location>
			<duration>16000</duration>
			<image>http://media.svt.se/download/mcc/flash/20090804/PR-2009-0804-SOBEKABUL/PR-2009-0804-SOBEKABUL_start_0.jpg</image>
		</track>
		<track>
			<title><![CDATA[12:08 En död i thailändsk flygkrasch]]></title>
			<location>http://www0.c00928.cdn.qbrick.com/00928/kluster/20090804/PR-2009-0804-SOBETHAIKRASCH.flv</location>
			<duration>17000</duration>
			<image>http://media.svt.se/download/mcc/flash/20090804/PR-2009-0804-SOBETHAIKRASCH/PR-2009-0804-SOBETHAIKRASCH_start_0.jpg</image>
		</track>
		<track>
			<title><![CDATA[10:16 Hamstring aav glödlampor inför EU-förbud]]></title>
			<location>http://www0.c00928.cdn.qbrick.com/00928/kluster/20090804/PR-2009-0804-GLODLAMPOR.flv</location>
			<duration>93000</duration>
			<image>http://media.svt.se/download/mcc/flash/20090804/PR-2009-0804-GLODLAMPOR/PR-2009-0804-GLODLAMPOR_start_0.jpg</image>
		</track>
		<track>
			<title><![CDATA[10:14 Miljardlån till vänner före Kaupthings krasch]]></title>
			<location>http://www0.c00928.cdn.qbrick.com/00928/kluster/20090804/PR-2009-0804-PEEDKAUPTHINGKARUSELL.flv</location>
			<duration>62000</duration>
			<image>http://media.svt.se/download/mcc/flash/20090804/PR-2009-0804-PEEDKAUPTHINGKARUSELL/PR-2009-0804-PEEDKAUPTHINGKARUSELL_start_0.jpg</image>
		</track>
		<track>
			<title><![CDATA[10:07 Hemliga polislistor över kriminella säljs]]></title>
			<location>http://www0.c00928.cdn.qbrick.com/00928/kluster/20090804/PR-2009-0804-PEEDPOLISLISTOR.flv</location>
			<duration>81000</duration>
			<image>http://media.svt.se/download/mcc/flash/20090804/PR-2009-0804-PEEDPOLISLISTOR/PR-2009-0804-PEEDPOLISLISTOR_start_0.jpg</image>
		</track>
		<track>
			<title><![CDATA[08:47 Röst på SD i kyrkovalet kan vara bortkastad]]></title>
			<location>http://www0.c00928.cdn.qbrick.com/00928/kluster/20090804/PR-2009-0804-SOBESDVAL.flv</location>
			<duration>96000</duration>
			<image>http://media.svt.se/download/mcc/flash/20090804/PR-2009-0804-SOBESDVAL/PR-2009-0804-SOBESDVAL_start_0.jpg</image>
		</track>
		<track>
			<title><![CDATA[08:37 Hård kritik mot svenska häkten]]></title>
			<location>http://www0.c00928.cdn.qbrick.com/00928/kluster/20090804/PR-2009-0804SOBEPALHAKTE.flv</location>
			<duration>20000</duration>
			<image>http://media.svt.se/download/mcc/flash/20090804/PR-2009-0804SOBEPALHAKTE/PR-2009-0804SOBEPALHAKTE_start_0.jpg</image>
		</track>
		<track>
			<title><![CDATA[08:15 Åratal av sanering efter oljekatastrofen]]></title>
			<location>http://www0.c00928.cdn.qbrick.com/00928/kluster/20090804/PR-2009-0804-SOBEOLJA.flv</location>
			<duration>33000</duration>
			<image>http://media.svt.se/download/mcc/flash/20090804/PR-2009-0804-SOBEOLJA/PR-2009-0804-SOBEOLJA_start_0.jpg</image>
		</track>
		<track>
			<title><![CDATA[01:33 Nya fall av resistenta patienter]]></title>
			<location>http://www0.c00928.cdn.qbrick.com/00928/kluster/20090804/PR-2009-0804-LISVPALSVININFLUENSANATT.flv</location>
			<duration>25000</duration>
			<image>http://media.svt.se/download/mcc/flash/20090804/PR-2009-0804-LISVPALSVININFLUENSANATT/PR-2009-0804-LISVPALSVININFLUENSANATT_start_0.jpg</image>
		</track>
		<track>
			<title><![CDATA[01:09 Inget VM för Christian Olsson]]></title>
			<location>http://www0.c00928.cdn.qbrick.com/00928/kluster/20090804/PR-2009-0804-LISVPALOLSSONNATT.flv</location>
			<duration>16000</duration>
		</track>
		<track>
			<title><![CDATA[00:24 Läkare missar ofta KOL]]></title>
			<location>http://www0.c00928.cdn.qbrick.com/00928/kluster/20090804/PR-2009-0804-LISVKOLNATT.flv</location>
			<duration>97000</duration>
			<image>http://media.svt.se/download/mcc/flash/20090804/PR-2009-0804-LISVKOLNATT/PR-2009-0804-LISVKOLNATT_start_0.jpg</image>
		</track>
		<track>
			<title><![CDATA[Igår 23:35 Kan bli ny rättegång för Quick]]></title>
			<location>http://www0.c00928.cdn.qbrick.com/00928/kluster/20090803/PR-2009-0804-LISVPALOUICK.flv</location>
			<duration>14000</duration>
		</track>
		<track>
			<title><![CDATA[Igår 23:32 Ljusning för Volvo, nattsvart för Saab]]></title>
			<location>http://www0.c00928.cdn.qbrick.com/00928/kluster/20090803/PR-2009-0804-LISVPALBILARNATT.flv</location>
			<duration>14000</duration>
		</track>
		<track>
			<title><![CDATA[Igår 23:30 Ojämn kamp mot opium i Afghanistan]]></title>
			<location>http://www0.c00928.cdn.qbrick.com/00928/kluster/20090803/PR-2009-0804-LISVOPIUMNATT.flv</location>
			<duration>108000</duration>
			<image>http://media.svt.se/download/mcc/flash/20090803/PR-2009-0804-LISVOPIUMNATT/PR-2009-0804-LISVOPIUMNATT_start_0.jpg</image>
		</track>
		<track>
			<title><![CDATA[Igår 23:24 Jacksons mamma får vårdnaden]]></title>
			<location>http://www0.c00928.cdn.qbrick.com/00928/kluster/20090803/PR-2009-0804-LISVJACKSONNATT.flv</location>
			<duration>58000</duration>
			<image>http://media.svt.se/download/mcc/flash/20090803/PR-2009-0804-LISVJACKSONNATT/PR-2009-0804-LISVJACKSONNATT_start_0.jpg</image>
		</track>
		<track>
			<title><![CDATA[Igår 23:21 Kapten misstänkt för brott vid oljekatastrof]]></title>
			<location>http://www0.c00928.cdn.qbrick.com/00928/kluster/20090803/PR-2009-0804-LISVNORGENATT.flv</location>
			<duration>72000</duration>
			<image>http://media.svt.se/download/mcc/flash/20090803/PR-2009-0804-LISVNORGENATT/PR-2009-0804-LISVNORGENATT_start_0.jpg</image>
		</track>
		<track>
			<title><![CDATA[Igår 18:33 Influensa-informatörer var tjuvar]]></title>
			<location>http://www0.c00928.cdn.qbrick.com/00928/kluster/20090803/PR-2009-0803-PEDGRUNE.flv</location>
			<duration>118000</duration>
			<image>http://media.svt.se/download/mcc/flash/20090803/PR-2009-0803-PEDGRUNE/PR-2009-0803-PEDGRUNE_start_0.jpg</image>
		</track>
		<track>
			<title><![CDATA[Igår 18:10 Ingen dykning efter kroppar]]></title>
			<location>http://www0.c00928.cdn.qbrick.com/00928/kluster/20090803/PR-2009-0803-PEDGDYKA.flv</location>
			<duration>79000</duration>
			<image>http://media.svt.se/download/mcc/flash/20090803/PR-2009-0803-PEDGDYKA/PR-2009-0803-PEDGDYKA_start_0.jpg</image>
		</track>
	</trackList>
</playlist>
